<?php

/**
 * Classes du service Miriade de calcul des ephemerides planetaires
 * @author J. Berthier (IMCCE/OBSPM) <berthier@imcce.fr>
 * @since PHP5
 * @version 2.0
 * @copyright (C) 2005-2013, IMCCE
 * @package MiriadeClasses
 * @subpackage Form
 */

date_default_timezone_set("UTC");
$error = FALSE;

$miriade = new callMiriadePaf();
// Collect user parameters and compute
if (($err = $miriade->getUserParams()) === TRUE) {
  $error = TRUE;
  echo $miriade->showError();
} else {
  if (($err = $miriade->compute()) === TRUE) {
    $error = TRUE;
    echo $miriade->showError();
  }
}

// Display results
if (!$error) {
  if (($result = $miriade->getResults()) === FALSE) {
    $error = TRUE;
    echo $miriade->showError();
  } else {
    echo $result;
  }
}


/**
 * Classe d'interface avec le Web service Miriade de calcul des ephemerides des Sso.
 * @package VOphpWebServices
 * @subpackage Miriade
 */
class callMiriadePaf
{

  /**
   * @var string URI of Miriade service
   */
  private $miriadeURI = 'http://vo.imcce.fr/webservices/miriade';
  /**
   * @var string WSDL of Miriade service
   */
  private $miriadeWSDL = 'http://vo.imcce.fr/webservices/miriade/miriade.wsdl';
  /**
   * @var string VOTable namespace
   */
  private $votableSchemaNS = 'http://www.ivoa.net/xml/VOTable/v1.1';
  
  /**
   * @var ArrayObject Array which contains the ephemeris parameters
   */
  private $param;
  /**
   * @var boolean true if an error occurs
   */
  private $error;
  /**
   * @var string Type of error (Error | Warning)
   */
  private $type_error;
  /**
   * @var string Error message
   */
  private $msg_error;
  /**
   * @var string String which contains the ephemeris result
   */
  private $results;

  /**
   * @var array Defines the code of reference planes
   */
  private $refplan = array('Equator'=>1,'Ecliptic'=>2);
  /**
   * @var array Defines the code of types of coordinates
   */
  private $typecoord = array('Spherical'=>1,'Rectangular'=>2,'Dedicated to obs.'=>5);
  /**
   * @var array Defines the code of types of ephemerides
   */
  private $typephem = array('AstrometricJ2000'=>1,'ApparentOfDate'=>2,'MeanOfDate'=>3,'MeanJ2000'=>4);
  /**
   * @var string Mime type of the response
   */
  private $mimeType;

  /**
   * Constructeur de la classe
   * @access public
   * @return void
   */
  public function __construct ()
  {
    $this->param = NULL;
    $this->error = FALSE;
    $this->type_error = '';
    $this->msg_error = '';
    $this->results = '';
    $this->mimeType = 'text';
  }

  /**
   * Collect the requested parameters of the ephemeris to compute 
   * @access public
   * @return boolean false if an error occurs, else true 
   */
  public function getUserParams ()
  {
    // Initialisations
    $this->error = FALSE;
    $this->type_error = '';
    $this->msg_error = '';

    // Si des parametres ont ete postes
    if (!empty($_POST)) {

      // ============================================================================================
      // Analyse des arguments : parametres de cible
      // ============================================================================================
      $target = '';
      $target_type = '';
      if (!empty($_POST['paf_body'])) {
        $ztarget = explode(",",$_POST['paf_body']);
        if (count($ztarget) > 1) {
          // Cas multi-target => un seul type de corps (aster ou planet ou comet)
          // Si le type de corps n'est pas donne => aster par defaut
        	$alltargets = '';
        	$alltypes = array();
        	// recupere les id et type des cibles
        	foreach ($ztarget as $z) {
        		$zu = explode(":",trim($z));
        		if (count($zu) > 1) {
        		  $alltypes[] = $zu[0];
        			$alltargets .= ucwords($zu[1]).", ";
        		} else {
        		  $alltypes[] = 'aster';
        		  $alltargets .= ucwords($zu[0]).", ";
        		} 
        	}
        	// verifie que les types sont equivalents
        	$target_type_error = false;
          $type = self::getTypeOfTarget($alltypes[0]);
        	foreach ($alltypes as $z) {
        	  $target_type_error = (self::getTypeOfTarget($z) != $type) ? true : false;
        	}
          if ($target_type_error) {
            $this->error = TRUE;
            $this->type_error = 'Error';
            $this->msg_error .= 'Bad request: target types cannot be mixed. All targets must be of same type (e.g. asteroid or comet or planet and natural satellite);';
          } else {
          	file_put_contents($this->targetFileName,$alltargets);
          	$target = $this->targetFileUrl;
          	$target_type = $type;
          }
        } else {
          // Cas target unique
          $target = $ztarget[0];
          $target_type = '';
        }
      } else {
        $this->error = TRUE;
        $this->type_error = 'Error';
        $this->msg_error .= 'Bad request: target cannot be empty or null;';
      }

      // ============================================================================================
      // Analyse des arguments : parametres temporels
      // ============================================================================================
      $epoch = '0';
      if (!empty($_POST['paf_epoch'])) {
        $postepoch = explode(",",$_POST['paf_epoch']);
        // -- Epoch
        if (($epoch = self::epochToJD(trim($postepoch[0]))) === FALSE) {
          $this->error = TRUE;
          $this->type_error = 'Error';
          $this->msg_error .= 'Bad request: start epoch '.(trim($postepoch[0])).' is not valid;';
        }
        // -- Epoch fin
        $postepochFin = explode(" ",trim($postepoch[1]));
        if (($epochFin = self::epochToJD(trim($postepochFin[0]))) === FALSE) {
          $this->error = TRUE;
          $this->type_error = 'Error';
          $this->msg_error .= 'Bad request: end epoch '.(trim($postepoch[0])).' is not valid;';
        }
        if (is_numeric($postepochFin[1])) {
          $epochFin += $postepochFin[1];
        }
        // -- Pas de calcul
        $cstep = explode("-",trim($postepoch[2]));
        $sval = $cstep[0];
        $unit = $cstep[1];
        $step = trim($sval).substr(trim($unit), 0, 1);
        // -- Nb dates de cacul
        switch ($unit) {
          case 'day':
            $ndate = abs($epochFin - $epoch) + 1;
            break;
          case 'hour':
            $ndate = abs($epochFin - $epoch)*24.0/$sval + 1;
            break;
          case 'minute':
            $ndate = abs($epochFin - $epoch)*1440.0/$sval + 1;
            break;
          default:
            $ndate = abs($epochFin - $epoch)*24.0/$sval + 1;
        }

        if ($ndate < 1 && $ndate > 5000) {
          $ndate = 1; // par defaut
        }
      } else {
        $this->error = TRUE;
        $this->type_error = 'Error';
        $this->msg_error .= 'Bad request: epoch cannot be empty or null;';
      }
   
      // ============================================================================================
      // Analyse des arguments : parametres de repere
      // ============================================================================================
      $refcenter = '';
      if (!empty($_POST['paf_refcenter'])) {
        $postrefcenter = trim($_POST['paf_refcenter']);
        switch (strtolower($postrefcenter)) {
          case "309":
          case "cerro paranal":
          case "cerro paranal [309]":
            $refcenter = "@309";
            break;

          case "809":
          case "la silla":
          case "la silla [809]":
            $refcenter = "@809";
            break;

          default:
            $this->error = TRUE;
            $this->type_error = 'Error';
            $this->msg_error .= 'Bad request: unknown observer location ('.$postrefcenter.');';
        }
      } else {
        $this->error = TRUE;
        $this->type_error = 'Error';
        $this->msg_error .= 'Bad request: observer location cannot be empty or null;';
      }

          // ============================================================================================
      // Analyse des arguments : parametres de repere
      // ============================================================================================
      $pafParams = '';
      if (!empty($_POST['paf_params'])) {
        $pafParams = '--pafParams('.trim($_POST['paf_params']).')';
      } else {
        $pafParams = '--pafParams(2.6,0.0)';
      }

      // ============================================================================================
      // Autres parametres
      // ============================================================================================
      $scale         = 'UTC';
      $theory        = 'INPOP';
      $postrefplan   = 'Equator';
      $posttypecoord = 'Spherical';
      $posttypephem  = 'AstrometricJ2000';

      // ============================================================================================
      // Arguments de la requete
      // ============================================================================================
      if (!$this->error) {

        $this->param = array( 'name'     => $target,
                              'type'     => $target_type,
                              'epoch'    => $epoch,
                              'nbd'      => $ndate,
                              'step'     => $step,
                              'tscale'   => $scale,
                              'observer' => $refcenter,
                              'theory'   => $theory,
                              'teph'     => $this->typephem[$posttypephem],
                              'tcoor'    => $this->typecoord[$posttypecoord],
                              'rplane'   => $this->refplan[$postrefplan],
                              'mime'     => $this->mimeType,
                              'output'   => $pafParams
                            );
      }

    } else {

      // Aucun parametre poste
      $this->error = TRUE;
      $this->type_error = 'Error';
      $this->msg_error .= 'Bad request: no parameter has been posted;';

    }
    return $this->error;

  }

  /**
   * Ephemeris computation: the method calls the Web service Miriade (http://vo.imcce.fr/webservices/miriade)
   * @access public
   * @return boolean vrai si pas d'erreur, sinon faux  
   */
  public function compute ()
  {
    // Initialisations
    $this->msg_error = '';
      
    if (!$this->error) {
      // Mise en cache ou non du WSDL du service
      ini_set('soap.wsdl_cache_enabled', 1);
      // Client's ID
      $from = 'MiriadePafWebForm';
      // Execution de la requete
      try {
        // Instantiation du client
        $client = new SoapClient($this->miriadeWSDL, array('exceptions'=>1));
        // Parametres de l'entete SOAP
        $header = array('from'=>$from, 'hostip'=>'');
        // Construction de l'entete SOAP
        $client->__setSoapHeaders(array(new SOAPHeader($this->miriadeURI, 'clientID', $header)));
        // Appel de la methode
        $response = $client->__soapCall('ephemcc',array($this->param));
        // Resultat
        if ($response->flag == 1) {
          $this->results = $response->result;
        } else {
          self::getErrorFromVotable($response->result);
        }
      } catch (SoapFault $fault) {
        $this->error = TRUE;
        $this->type_error = 'Error';
        $this->msg_error = "SOAP Fault: {$fault->getTraceAsString()} (faultcode: {$fault->faultcode}, faultstring: {$fault->faultstring})";
      }
    }
    return $this->error;
  }

  /**
   * Impression des resultats
   * @access public
   * @param string $context contexte d'affichage des resultats (xhtml, xml, ascii)
   * @return resultats a afficher ou un message d'erreur  
   */
  public function getResults ()
  {
    $m = '';
    if (!empty($this->results)) {
      $r = str_replace(";", "\n", $this->results);
      $m = "<textarea>".$r."</textarea>";
      return $m;
    } else {
      $this->error = TRUE;
      $this->type_error = 'Error';
      $this->msg_error = 'No result to display';
      return FALSE;
    }
  }

  /**
  * Impression d'un msg d'erreur
  * @access public
  * @return message d'erreur a afficher
  */
  public function showError ()
  {
     $m = '';
     if (!empty($this->msg_error)) {
       $m  = '<fieldset id="error" class="'.strtolower($this->type_error).'">';
       $m .= '<legend><a class="msg_error"><img src="/webservices/share/img/close_trans.png" alt="Close"/></a></legend>';
       $m .= '<div class="msg">';
       $m .= (strtolower($this->type_error) == 'error') ? 'Error:' : 'Warning:';
       $m .= '<p>';
       $emsg = explode(';',$this->msg_error);
       foreach ($emsg as $cle=>$val) {
         if (strlen(trim($val)) > 0) {
           $m .= '- '.$val.'<br/>';
         }
       }
       $m .= '</p></div></fieldset>';
     }
     return $m;
  }

   /**
    * Provides the type of target from its code
    */
  private function getTypeOfTarget ($t)
  {
     if (strpos(strtolower($t), 'a') !== FALSE) { return 'aster';  }
     if (strpos(strtolower($t), 'p') !== FALSE) { return 'planet'; }
     if (strpos(strtolower($t), 's') !== FALSE) { return 'planet'; }
     if (strpos(strtolower($t), 'c') !== FALSE) { return 'comet';  }
     return 'aster';
   }

  /**
   * Recupere le statut et le message dans une VOTable
   * @return void 
   */
  private function getErrorFromVotable ($vot) 
  {
    try 
    {
      // Doc a analyser
      $xml = new SimpleXMLElement($vot, NULL, FALSE, $this->votableSchemaNS, FALSE);
      // Recherche des elements INFO
      $i = 0;
      $info = array();
      foreach ($xml->children($this->votableSchemaNS) as $child=>$node) {
        if ((string) $child == 'INFO') {
          foreach ($node->attributes() as $attr=>$val) {
            $info[$i][(string) $attr] = (string) $val;
          }
          $i++;
        }
      }
      // Recherche d'une erreur ou d'un warning
      foreach ($info as $infoKey=>$infoVal) {
        if (array_key_exists('ID',$infoVal) and array_key_exists('value',$infoVal)) {
          if (strtolower($infoVal['ID']) == 'status') {
            if (strtolower($infoVal['value']) == 'error' || strtolower($infoVal['value']) == 'warning') {
              $this->error = TRUE;
              $this->type_error = strtolower($infoVal['value']);
              $this->msg_error = $xml->INFO;
            }
          }
        }
      }
    } catch (Exception $e) {
      $this->error = TRUE;
      $this->type_error = 'Error';
      $this->msg_error = $e;
    }
  }

  /**
   * Conversion de la date calendaire en Jour Julien
   * @access public
   * @param  int   $M  mois
   * @param  int   $J  jour
   * @param  int   $A  annee
   * @param  float $H  heure
   * @return float Jour Julien
   */
  private function MyGregorianToJD ($M, $J, $A, $H)
  {
    $Date = $A + $M/12.0 + $J/365.25;
    if ($M < 3) {
      $Ry = $A -  1.0;
      $Rm = $M + 12.0;
    } else {
      $Ry = $A;
      $Rm = $M;
    }
    $Ra = floor($Ry/100.0);
    $Rb = 2.0 - $Ra + floor($Ra/4.0);
    if ($Date < (1582.0+10.0/12.0+5.0/365.25)) $Rb = 0.0;
    $Dd = $J + $H/24.0;
    $JJ = floor(365.25*($Ry+4716.0)) + floor(30.6001*($Rm+1.0)) + $Dd + $Rb - 1524.5;
    return $JJ;
  }

  /**
   * Transforme une epoque (now, iso, ...) en un jour julien
   * @access public
   * @param string $epoch une epoque (e.g. now, 2010-09-07T12:32:00, ...)
   * @return float le jour julien correspondant, FALSE sinon
   */
  private function epochToJD ($epoch)
  {
    if (!empty($epoch)) {
      $sub_epoch = $epoch;
      // Si l'epoque est un JD alors c bon
      if (preg_match("/^([[:digit:]]+)\.?([[:digit:]]*)$/", $sub_epoch, $matches)) {
        $jd = $sub_epoch;
      } else {
        if (preg_match("/^([[:digit:]-]+)T([[:digit:]:]+)$/", $sub_epoch, $matches)) {
          $sub_epoch = $matches[1]." ".$matches[2];
        }
        if (($timestamp = strtotime($sub_epoch)) === FALSE) {
          if (is_numeric($sub_epoch)) {
            $jd = $sub_epoch;
          } else {
            if (($toto = strstr($sub_epoch, ".")) != '') {
              return FALSE;
            } else {
              return FALSE;
            }
          }
        } else {
          // correction TL->UTC pour une date 'now'
          if ($sub_epoch == 'now') $timestamp = $timestamp-Time::tzdelta(strtotime($sub_epoch));
          // conversion de la date en JD
          $date = getDate ( $timestamp );
          $Hd = $date ['hours'] + ($date ['minutes'] + $date ['seconds'] / 60.0) / 60.0;
          $jd = self::MyGregorianToJD ( $date ['mon'], $date ['mday'], $date ['year'], $Hd );
        }
      }
      return $jd;
    } else {
      return FALSE;
    }
  }
 
}

?>
